"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf3 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _l7Core = require("@antv/l7-core");

var _l7Utils = require("@antv/l7-utils");

var _BaseLayer2 = _interopRequireDefault(require("../core/BaseLayer"));

var _triangulation = require("../core/triangulation");

var rasterImageFrag = "precision mediump float;\nuniform float u_opacity: 1.0;\nuniform sampler2D u_texture;\nuniform sampler2D u_colorTexture;\nuniform float u_min;\nuniform float u_max;\nvarying vec2 v_texCoord;\n\nvoid main() {\n\n  float value = texture2D(u_texture,vec2(v_texCoord.x,v_texCoord.y)).a;\n   value = clamp(value,u_min,u_max);\n  float value1 =  (value - u_min) / (u_max -u_min);\n  vec2 ramp_pos = vec2(\n        fract(16.0 * (1.0 - value1)),\n        floor(16.0 * (1.0 - value1)) / 16.0);\n  gl_FragColor =  texture2D(u_colorTexture,ramp_pos);;\n}\n";
var rasterImageVert = "precision highp float;\nuniform mat4 u_ModelMatrix;\nattribute vec3 a_Position;\nattribute vec2 a_Uv;\nvarying vec2 v_texCoord;\n#pragma include \"projection\"\nvoid main() {\n   v_texCoord = a_Uv;\n   vec4 project_pos = project_position(vec4(a_Position, 1.0));\n   gl_Position = project_common_position_to_clipspace(vec4(project_pos.xy,0., 1.0));\n}\n";

var Raster2dLayer = function (_BaseLayer) {
  (0, _inherits2.default)(Raster2dLayer, _BaseLayer);

  function Raster2dLayer() {
    var _getPrototypeOf2;

    var _this;

    (0, _classCallCheck2.default)(this, Raster2dLayer);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = (0, _possibleConstructorReturn2.default)(this, (_getPrototypeOf2 = (0, _getPrototypeOf3.default)(Raster2dLayer)).call.apply(_getPrototypeOf2, [this].concat(args)));
    _this.type = 'RasterLayer';
    _this.texture = void 0;
    _this.colorTexture = void 0;
    return _this;
  }

  (0, _createClass2.default)(Raster2dLayer, [{
    key: "getConfigSchema",
    value: function getConfigSchema() {
      return {
        properties: {
          opacity: {
            type: 'number',
            minimum: 0,
            maximum: 1
          }
        }
      };
    }
  }, {
    key: "renderModels",
    value: function renderModels() {
      var _this2 = this;

      var _this$getLayerConfig = this.getLayerConfig(),
          opacity = _this$getLayerConfig.opacity;

      var parserDataItem = this.getSource().data.dataArray[0];
      var min = parserDataItem.min,
          max = parserDataItem.max;

      if (this.texture) {
        this.models.forEach(function (model) {
          return model.draw({
            uniforms: {
              u_opacity: opacity || 1,
              u_texture: _this2.texture,
              u_min: min,
              u_max: max,
              u_colorTexture: _this2.colorTexture
            }
          });
        });
      }

      return this;
    }
  }, {
    key: "buildModels",
    value: function buildModels() {
      this.registerBuiltinAttributes();
      var source = this.getSource();
      var createTexture2D = this.rendererService.createTexture2D;
      var parserDataItem = this.getSource().data.dataArray[0];
      this.texture = createTexture2D({
        data: parserDataItem.data,
        width: parserDataItem.width,
        height: parserDataItem.height,
        format: _l7Core.gl.LUMINANCE,
        type: _l7Core.gl.FLOAT,
        aniso: 4
      });

      var _this$getLayerConfig2 = this.getLayerConfig(),
          rampColors = _this$getLayerConfig2.rampColors;

      var imageData = (0, _l7Utils.generateColorRamp)(rampColors);
      this.colorTexture = createTexture2D({
        data: imageData.data,
        width: imageData.width,
        height: imageData.height,
        flipY: true
      });
      this.models = [this.buildLayerModel({
        moduleName: 'Raster3DImage',
        vertexShader: rasterImageVert,
        fragmentShader: rasterImageFrag,
        triangulation: _triangulation.RasterImageTriangulation,
        primitive: _l7Core.gl.TRIANGLES,
        depth: {
          enable: false
        },
        blend: {
          enable: true,
          func: {
            srcRGB: _l7Core.gl.SRC_ALPHA,
            srcAlpha: 1,
            dstRGB: _l7Core.gl.ONE_MINUS_SRC_ALPHA,
            dstAlpha: 1
          }
        }
      })];
    }
  }, {
    key: "registerBuiltinAttributes",
    value: function registerBuiltinAttributes() {
      this.styleAttributeService.registerStyleAttribute({
        name: 'uv',
        type: _l7Core.AttributeType.Attribute,
        descriptor: {
          name: 'a_Uv',
          buffer: {
            usage: _l7Core.gl.DYNAMIC_DRAW,
            data: [],
            type: _l7Core.gl.FLOAT
          },
          size: 2,
          update: function update(feature, featureIdx, vertex, attributeIdx) {
            return [vertex[3], vertex[4]];
          }
        }
      });
    }
  }]);
  return Raster2dLayer;
}(_BaseLayer2.default);

exports.default = Raster2dLayer;
//# sourceMappingURL=raster2d.js.map